# -*- coding:utf-8 -*-
# @time:2022/12/2716:00
# @author:LX
# @file:comboBox.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QThread,
    Signal,
    QLineEdit,
    QInputMethodEvent,
    QKeyEvent,
    QPaintEvent,
    QMouseEvent,
    QPainter,
    QPen,
    QFont,
    QResizeEvent,
    QRect,
    QBrush,
    QColor,
    QScrollArea,
    QVBoxLayout,
    QPushButton,
    QLabel,
    Qt,
    QPropertyAnimation,
    QPoint,
    QSize
)


'''
    下拉框
'''


class LineEdit(QLineEdit):

    clicked = Signal() # 点击事件

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.setStyleSheet('''
border:none;
        ''')

        # 图标大小
        self.icon_w, self.icon_h = 12, 12
        # 图标位置靠右的位置,距离边的距离
        self.icon_right_margin = 25
        # 圆角
        self.icon_radius = (6,6)
        self.icon_color = QColor(0, 255, 0)

        # 动画开关
        self.animation_switch = False

        # self.startTimer(100)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.clicked.emit()
        self.animation_switch = True
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e: QMouseEvent) -> None:
        super().mouseReleaseEvent(e)

    # 绘制图标
    def drawIcon(self,painter:QPainter):
        parent = self.parent() # 父类对象
        h = self.height()//2-self.icon_h//2
        rect = QRect(parent.width() - self.icon_right_margin, h, self.icon_w,self.icon_h)

        op = QPen()
        op.setColor(self.icon_color)
        bush = QBrush(self.icon_color)

        painter.setPen(op)
        painter.setBrush(bush)

        painter.drawRoundedRect(rect,*self.icon_radius)

    # def timerEvent(self, e) -> None:
    #     self.update()



    def paintEvent(self, e: QPaintEvent) -> None:
        super().paintEvent(e)
        painter = QPainter(self)
        painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)

        # 绘制图标
        self.drawIcon(painter)

        painter.end()


class ScrollArea(QScrollArea):

    # 风格
    Style_Card = "card"  # 卡片

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.resize(500,500)

        self.setStyleSheet('''
        border:none;
                ''')

        # 风格模式
        self.style_mode = ScrollArea.Style_Card

        self.setWidgetResizable(True)
        self.core_widget = QWidget()
        self.setWidget(self.core_widget)

        # 隐藏滚动条
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        # 所有对象
        self.obj = []

        self.__vlay = QVBoxLayout(self.core_widget)
        self.__vlay.setContentsMargins(3,5,3,5)
        self.__vlay.setSpacing(3)

        self.item_fixed_height = 40

    # 卡片风格颜色
    def styleCrad(self)->str:
        style ='''
background-color: #237eff;
border:1px solid #237eff;
border-radius:5px;
color:#ffffff;
        '''
        return style

    def addWidget(self,widget:QWidget):
        if isinstance(widget,str):
            # text = widget
            widget_ = QLabel(widget)


            widget_.setMinimumHeight(self.item_fixed_height)
            # widget.setAlignment(Qt.AlignCenter)
            if self.style_mode == ScrollArea.Style_Card:
                widget_.setStyleSheet(self.styleCrad())
        self.__vlay.addWidget(widget_)
        self.obj.append(widget_)

    def itemChildWidget(self)->list:
        return self.obj

    def itemCount(self)->int:
        return len(self.obj)


class ComboBox(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.w,self.h = 400,40
        self.resize(self.w,self.h)
        self.raise_()

        self.setStyleSheet('font: 11pt "华文细黑";')

        # 展开区域,大小,位置
        self.EA_w = self.w
        self.EA_h = 200
        self.EA_x = 0
        self.EA_y = self.h
        self.EA_area_max_height = 200 # 展开区域默认的最大高度
        self.old_h = self.h  # 保存旧的高度
        self.EA_state = False # 状态,展开,隐藏

        self.EA_up_margin = 0  # 下拉框与选择区域的距离

        # 单个子项高度
        self.item_height = 40

        # 展开区域的伸缩动画持续时间
        self.EA_stretch_ani_duration_time = 400
        # 点击事件标记
        self.click_flag = True

        # 创建选择区域
        self.createcChooseArea()
        # 创建展开区域
        self.createExpandArea()

        self.test()

    def test(self):
        self.setEAUpMargin(0)
        for i in range(10):
            self.addText("选项{}".format(str(i)))

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        super().resize(self.w,self.h)

    # 添加文本
    def addText(self,text:str):
        self.EA_widget.addWidget(text)

    # 创建选择区域
    def createcChooseArea(self):
        self.__choose_area = LineEdit(self)
        self.__choose_area.move(0, 0)
        self.__choose_area.resize(self.w, self.h)
        self.__choose_area.clicked.connect(self.expand_event)

    # 设置下拉框与选择区域的距离
    def setEAUpMargin(self,margin:int):
        self.EA_up_margin = margin

    def setItemHeight(self,h:int):
        self.item_height = h

    # 设置展开区域的伸缩动画持续时间
    def setEAStretchAniDuration(self,msecs:int):
        self.EA_stretch_ani_duration_time = msecs

    # 创建展开区域
    def createExpandArea(self):
        self.EA_widget = ScrollArea(self)
        self.EA_widget.move(self.EA_x,self.EA_y)
        self.EA_widget.resize(self.EA_w,self.EA_h+self.EA_up_margin)

    # 展开事件
    def expand_event(self):
        if self.EA_state is False:
            self.old_h = self.h
            self.click_flag = False
            self.stretchAni()
            self.EA_state = True
        else:
            self.stretchAni()
            self.EA_state = False

    # 伸缩动画
    def stretchAni(self):
        ani_expand = QPropertyAnimation(self)
        ani_expand.setPropertyName(b"size")
        ani_expand.setTargetObject(self)
        ani_expand.setStartValue(self.size())

        if self.EA_state is False:
            # item个数 * 单个大小 = 总高度
            area_height = self.EA_widget.itemCount() * self.item_height
            if area_height > self.EA_area_max_height:
                area_height = self.EA_area_max_height
            ani_expand.setEndValue(QSize(self.width(),self.height()+area_height))
        else:
            ani_expand.setEndValue(QSize(self.__choose_area.width(),self.__choose_area.height()))

        ani_expand.setDuration(self.EA_stretch_ani_duration_time)
        ani_expand.start()

    def resizeEvent(self, e: QResizeEvent) -> None:
        self.w, self.h = e.size().width(),e.size().height()

        if self.click_flag:
            self.__choose_area.resize(self.w, self.h)
        else:
            self.click_flag = False
            self.__choose_area.resize(self.w,self.old_h)

        #
        self.EA_y = self.h
        self.EA_w = self.w
        self.EA_widget.move(self.EA_x, self.old_h+self.EA_up_margin)
        self.EA_widget.resize(self.EA_w,self.h-self.old_h-self.EA_up_margin)
        super().resizeEvent(e)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = ComboBox()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    Qt,
    QWidget,
    QGridLayout,
    QLabel,
    QFont
)
'''
    亚克力窗口
'''

from windowEffect import WindowEffect

from PyQtGuiLib.core.widgets import BorderlessWidget

class AcrylicWidget(BorderlessWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,600)

        # self.setWindowFlags(Qt.FramelessWindowHint)  # make the window frameless
        self.setAttribute(Qt.WA_TranslucentBackground)  # make the window translucent
        #
        self.ui_layout = QGridLayout(self)  # create a ui layout
        self.ui_layout.setAlignment(Qt.AlignCenter)  # center the layout

        self.label = QLabel("Hello World!", self)  # create a label to display a text
        self.label.setFont(QFont("Segoe UI", 14))  # configure the text size and font
        # self.ui_layout.addWidget(self.label)  # add the label widget into the layout
        self.windowFX = WindowEffect()  # instatiate the WindowEffect class
        self.windowFX.setAcrylicEffect(self.winId())  # set the Acrylic effect by
        # self.windowFX.setAeroEffect(self.winId())

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = AcrylicWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header.versions import PYQT_VERSIONS

if PYQT_VERSIONS == "PyQt5":
    from PyQt5.QtCore import (
        Qt,
        QUrl,
        pyqtSignal as Signal,
        QSize,
        QPoint,
        QPointF,
        QUrl,
        QRectF,
        QRect,
        QPropertyAnimation,
        QThread,
        QEasingCurve,
        QByteArray,
        QSequentialAnimationGroup,
        QParallelAnimationGroup
    )

if PYQT_VERSIONS == "PyQt6":
    from PyQt6.QtCore import (
        Qt,
        QUrl,
        pyqtSignal as Signal,
        QSize,
        QPoint,
        QPointF,
        QUrl,
        QRectF,
        QRect,
        QPropertyAnimation,
        QThread,
        QEasingCurve,
        QByteArray,
        QSequentialAnimationGroup,
        QParallelAnimationGroup
    )

if PYQT_VERSIONS == "PySide2":
    from PySide2.QtCore import (
        Qt,
        QUrl,
        Signal,
        QSize,
        QPoint,
        QPointF,
        QUrl,
        QRectF,
        QRect,
        QPropertyAnimation,
        QThread,
        QEasingCurve,
        QByteArray,
        QSequentialAnimationGroup,
        QParallelAnimationGroup
    )
if PYQT_VERSIONS == "PySide6":
    from PySide6.QtCore import (
        Qt,
        QUrl,
        Signal,
        QSize,
        QPoint,
        QPointF,
        QUrl,
        QRectF,
        QRect,
        QPropertyAnimation,
        QThread,
        QEasingCurve,
        QByteArray,
        QSequentialAnimationGroup,
        QParallelAnimationGroup
    )


# -*- coding:utf-8 -*-
# @time:2022/12/1711:39
# @author:LX
# @file:borderlessWidget.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    Qt,
    QMouseEvent,
    QPoint,
    QMainWindow,
    QFrame,
    QStackedWidget
)

'''
    无边框窗口
        可自由拉伸八个方位
    
    目前支持的原型窗口
    QMainWindow
    QWidget
    QFrame,
    QStackedWidget
'''


# 无边框窗口移动的提示实现类
class Borderless:
    Top = "top"
    Down = "Down"
    Left = "Left"
    Right = "Right"

    def __init__(self,*args,**kwargs):
        self.scope = 8  # 检测鼠标是否在边缘按下的范围
        self.pressDirection = []  # 记录鼠标点击的边的方向
        self.pressPos = QPoint(0, 0)  # 鼠标按下时的位置
        self.pressState = False  # 按下状态
        self.movePressState = False  # 记录移动时是否时按下的状态

    # 边缘检测
    def isEdge(self,parent:QWidget,pos:QPoint):
        x,y = pos.x(),pos.y()
        w,h = parent.width(),parent.height()

        if x <= self.scope and x >= 0:
            self.pressDirection.append(Borderless.Left)
        if w - x <= self.scope:
            self.pressDirection.append(Borderless.Right)
        if y >= 0 and y <= self.scope:
            self.pressDirection.append(Borderless.Top)
        if h - y <= self.scope:
            self.pressDirection.append(Borderless.Down)

        if self.pressDirection:
            return True

    # 更新鼠标的图标
    def updateCursor(self,parent:QWidget, pos: QPoint):
        x, y = pos.x(), pos.y()
        w, h = parent.width(), parent.height()
        '''
            先判断四角,再判断四边
        '''
        if (x <= self.scope and x >= 0) and (y >= 0 and y <= self.scope) \
                or (w - x <= self.scope) and (h - y <= self.scope):
            parent.setCursor(Qt.SizeFDiagCursor)
        elif (w - x <= self.scope) and (y >= 0 and y <= self.scope) \
                or (x <= self.scope and x >= 0) and (h - y <= self.scope):
            parent.setCursor(Qt.SizeBDiagCursor)
        elif (x <= self.scope and x >= 0) or (w - x <= self.scope):
            parent.setCursor(Qt.SizeHorCursor)
        elif (y >= 0 and y <= self.scope) or (h - y <= self.scope):
            parent.setCursor(Qt.SizeVerCursor)

        else:
            parent.setCursor(Qt.ArrowCursor)

        if self.pressDirection:
            return True

    # 扩展边
    def expandEdge(self,parent:QWidget,pos:QPoint):
        if not self.pressState:
            return

        x,y = parent.pos().x(),parent.pos().y()
        for direction in self.pressDirection:
            if direction == Borderless.Right:
                if Borderless.Down not in self.pressDirection:
                    distance = pos - self.pressPos
                    distance_ = parent.width() + distance.x()
                    parent.resize(distance_, parent.height())
                    self.pressPos = QPoint(distance_, 0)
            if direction == Borderless.Down:
                distance = pos - self.pressPos
                distance_ = parent.height() + distance.y()
                if Borderless.Right in self.pressDirection:
                    distance_w = parent.width() + distance.x()
                    parent.resize(distance_w, distance_)
                    self.pressPos = QPoint(distance_w, distance_)
                else:
                    parent.resize(parent.width(),distance_)
                    self.pressPos = QPoint(0,distance_)
            if direction == Borderless.Left:
                distance_ = x + pos.x()
                parent.resize(parent.width()-pos.x(),parent.height())
                parent.move(distance_, y)
            if direction == Borderless.Top:
                distance_ = y + pos.y()
                parent.resize(parent.width(), parent.height()-pos.y())
                if Borderless.Left in self.pressDirection:
                    x = x+pos.x()
                parent.move(x,distance_)

    def pressEvent(self,parent:QWidget, e:QMouseEvent) -> None:
        if self.isEdge(parent,e.pos()):
            self.pressState = True
            self.pressPos = e.pos()
        elif e.button() == Qt.LeftButton:  # 处理窗口移动
            self.movePressState = True
            self.pressPos = e.globalPos()-parent.pos()
            parent.setCursor(Qt.OpenHandCursor)

    def releaseEvent(self) -> None:
        self.pressDirection.clear()
        self.pressPos = QPoint(0, 0)
        self.pressState = False
        self.movePressState = False

    def moveEvent(self,parent:QWidget, e:QMouseEvent) -> None:
        if self.movePressState:
            parent.move(e.globalPos()-self.pressPos)
        self.updateCursor(parent,e.pos())
        self.expandEdge(parent,e.pos())


# 无边框的主窗口
class BorderlessMainWindow(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800, 500)

        self.borderless = Borderless()

        self.setWindowFlags(Qt.FramelessWindowHint)
        # 开启鼠标跟踪
        self.setMouseTracking(True)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)


# 无边框的QWidget
class BorderlessWidget(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800, 500)

        self.borderless = Borderless()

        self.setWindowFlags(Qt.FramelessWindowHint)
        # 开启鼠标跟踪
        self.setMouseTracking(True)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)


# 无边框的QFrame
class BorderlessFrame(QFrame):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800, 500)

        self.borderless = Borderless()

        self.setWindowFlags(Qt.FramelessWindowHint)
        # 开启鼠标跟踪
        self.setMouseTracking(True)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)


# 无边框的QStackedWidget
class BorderlessStackedWidget(QStackedWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800, 500)

        self.borderless = Borderless()

        self.setWindowFlags(Qt.FramelessWindowHint)
        # 开启鼠标跟踪
        self.setMouseTracking(True)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = BorderlessStackedWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/2710:45
# @author:LX
# @file:gradientBar.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    Signal,
    Qt,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent,
    QLinearGradient,
    QBrush,
    Signal
)
'''
    线性渐变进度条
'''

class GradientBar(QWidget):
    # 进度改变时,发出信号
    valueChange = Signal(int)
    
    def __init__(self,*args,**kwargs):
        self.max_value = 100 # 进度条最大百分比值
        super().__init__(*args,**kwargs)

        self.w,self.h = 800,50
        self.resize(self.w,self.h)

        self.degree = round(self.w / self.max_value, 2)

        # 进度条真实长度
        self.bw = 0

        # 当前进度条的百分比值
        self.cu_value = 0

        # 渐变颜色列表
        '''
            [(颜色比重0-1,QColor()),...]
        '''
        self.line_color_lists = []

        # 圆角半径
        self.radius = 0

        self.setValue(100)

        # 进度条底色
        self.bg_color = QColor(211, 211, 211)

        self.setColorAts(
            [(0.2,QColor(170, 170, 255)),(0.4, QColor(170, 255, 127)),(0.6, QColor(85, 170, 127))]
             )

    # 设置颜色比重和颜色
    def setColorAts(self,colors:list):
        '''

        :param colors:  [(颜色比重0-1,QColor()),...]
        :return:
        '''
        self.line_color_lists = colors

    # 设置背景底色
    def setBackGroundColor(self,bg_color:QColor):
        self.bg_color = bg_color

    # 添加一种颜色
    def appendColor(self,color:tuple):
        '''

        :param color: (颜色比重0-1,QColor())
        :return:
        '''
        self.line_color_lists.append(color)

    # 移除一种颜色
    def removeColor(self,color:tuple):
        '''

        :param color: (颜色比重0-1,QColor())
        :return:
        '''
        self.line_color_lists.remove(color)

    # 返回所有的颜色和比重
    def getColors(self) -> list:
        return self.line_color_lists

    def setValue(self,value:int):
        self.cu_value = value
        self.bw = int(self.degree*value)
        
        self.update()
        self.valueChange.emit(value)
        
    def value(self) -> int:
        return self.cu_value

    # 设置圆角半径
    def setRadius(self,r:int):
        self.radius = r

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        self.degree = round(self.w/self.max_value,2)
        super().resize(self.w,self.h)

    # 渐变进度条
    def gradientBar(self,painter:QPainter):
        # 进度条颜色渐变
        gradient = QLinearGradient(0, 0, self.w, self.h)
        for proportion,color in self.line_color_lists:
            gradient.setColorAt(proportion,color)

        # 进度条底色
        bg = QBrush(self.bg_color)

        painter.setPen(Qt.NoPen)  # 设置无画笔
        painter.setBrush(bg)
        # painter.drawRect(0, 0, self.w, self.h)
        painter.drawRoundedRect(0, 0, self.w, self.h,self.radius,self.radius)
        painter.setBrush(gradient)
        # painter.drawRect(0, 0, self.bw, self.h)
        painter.drawRoundedRect(0, 0, self.bw, self.h, self.radius,self.radius)

    def paintEvent(self, e:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)

        # 渐变进度条
        self.gradientBar(painter)

        painter.end()

    def resizeEvent(self, e:QResizeEvent) -> None:
        self.w = e.size().width()
        self.h = e.size().height()

        self.degree = round(self.w/self.max_value,2)
        self.setValue(self.value())
        super().resizeEvent(e)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = GradientBar()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    Qt,
    QMouseEvent,
    Signal
)

from PyQtGuiLib.core.widgets import RoundWidget


'''
    可点击的窗口
'''
class ButtonWidget(RoundWidget):
    clicked = Signal()

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        if e.button() == Qt.LeftButton:
            self.clicked.emit()
        super().mouseReleaseEvent(e)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = ButtonWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/296:18
# @author:LX
# @file:test_comboBox.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor,
    QThread,
    Qt,
    QPushButton
)

from PyQtGuiLib.core.comboBox import ComboBox

'''
    下拉框 测试用例
'''

class TestComboBox(QMainWindow):
    sed = Signal()
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.combox = ComboBox(self)
        # self.combox.setFocus()
        self.combox.resize(300,40)
        self.combox.move(100,100)
        print(self.combox.pos())

    def test(self,e):
        print(e)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestComboBox()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/2215:14
# @author:LX
# @file:test_loadbar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor
)

from PyQtGuiLib.core.progressBar import LoadBar


class DurationTimeThread(QThread):
    added = Signal(int)
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


    def run(self) -> None:
        n=1
        while True:
            self.added.emit(n)
            n+=1
            self.msleep(100)
            if n == 100+1:
                print("完成")
                break

class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.loadbar = LoadBar(self)
        self.loadbar.move(50,100)
        self.loadbar.resize(350,45)
        self.loadbar.setValue(0)

        self.th = DurationTimeThread()
        self.th.added.connect(self.test)
        self.th.start()

    def test(self,v):
        print(v)
        self.loadbar.setValue(v)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2022/12/2216:59
# @author:LX
# @file:waterBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QWidget,
    Signal,
    Qt,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent,
    QPropertyAnimation,
    QPoint,
    QThread
)

'''
    水球进度条
'''
from random import randint

# 小气泡
class SmallBubble(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.w,self.h = 15,15

        self.ani = QPropertyAnimation(self)
        self.ani.setPropertyName(b"pos")
        self.ani.setTargetObject(self)
        self.ani.finished.connect(self.close)

    # 球运行的速度区间
    def setDuration(self,interval:list=[1200,4000]):
        self.ani.setDuration(randint(*interval))

    def setStartValue(self,pos:QPoint):
        self.ani.setStartValue(pos)

    def setEndValue(self,pos:QPoint):
        self.ani.setEndValue(pos)
        self.ani.start()

    # 随机产生气泡
    def randomBubble(self,painter:QPainter):
        painter.setPen(QPen(QColor(255,255,255),2))
        painter.drawEllipse(2, 2, self.w, self.h)

    def paintEvent(self, e: QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)

        self.randomBubble(painter)
        painter.end()


class WaterBar(QWidget):
    # 进度改变时,发出信号
    valueChange = Signal(int)

    def __init__(self,*args,**kwargs):
        self.ball_interval = [1, 1] # 每次数值改变产生球的数量区间
        self.speed_interval = [1200,4000] # 小球运动的速度区间
        self.size_interval = [5,15] # 小球大小区间

        super().__init__(*args,**kwargs)

        self.w,self.h = 200,200
        self.resize(self.w,self.h)

        self.sn = 180
        self.en = 3.6

        self.water_vat_color = QColor(61, 203, 255,255) # 水缸里面没有被水覆盖的颜色
        self.water_vat_border_color = QColor(21, 185, 255,180) # 水缸边的颜色
        self.water_color = QColor(0, 135, 203) # 水的颜色

        # 当前百分比
        self.cu_value = 0

        # 文字属性
        self.text = "100%"
        self.text_size = 40
        self.text_color = QColor(0, 188, 188)
        self.__is_hide_text = False

    # 产生气泡
    def createBubble(self):
        for i in range(randint(*self.ball_interval)):
            rondomw = randint(*self.size_interval)
            sm = SmallBubble(self)
            sm.setDuration(self.speed_interval)
            sm.show()
            sm.w,sm.h= rondomw,rondomw
            sm.move(randint(self.w//2-self.w//4,self.w//2-self.w//10),
                    self.h-randint(20,40))
            sm.setStartValue(sm.pos())
            sm.setEndValue(QPoint(self.w//2-randint(10,40),
                                  self.h//2-randint(10,40)))

    def setValue(self,v:int):
        self.cu_value = v
        self.text = "{}%".format(v)
        self.update()
        self.valueChange.emit(v)
        self.createBubble()

    def setText(self,text:str):
        self.text = text

    def setTextColor(self,color:QColor):
        self.text_color = color

    def setTextSize(self,size:int):
        self.text_size = size

    def setAllText(self,text:str,color:QColor,size:int):
        self.setText(text)
        self.setTextColor(color)
        self.setTextSize(size)

    def isHideText(self,b:bool):
        self.__is_hide_text = b

    # 设置每个数值变化,球产生的个数区间
    def setBallInterval(self,interval:list):
        self.ball_interval = interval

    # 设置每颗球移动的速度区间
    def setBallSpeedInterval(self,interval:list):
        self.speed_interval = interval

    # 设置每颗球生成的大小区间
    def setBallSizeInterval(self,interval:list):
        self.size_interval = interval

    # 设置水的颜色
    def setWaterColor(self,color:QColor):
        self.water_color = color

    # 设置水缸中没有被水覆盖的颜色
    def setWaterVatColor(self,color:QColor):
        self.water_vat_color = color

    # 设置水缸边缘的颜色
    def setWaterVatBorderColor(self,color:QColor):
        self.water_vat_border_color = color

    # 画水位
    def drawWaterLevel(self,painter:QPainter):
        painter.setPen(QPen(self.water_vat_border_color, 3))
        # 画圆
        painter.setBrush(self.water_vat_color)
        painter.drawEllipse(2,2,self.w-4,self.h-4)

        # --------------
        # painter.drawChord(QtCore.QRect(self.upBlueXPoint, self.upBlueYPoint, self.radius * 2,
        #                                self.radius * 2), (-self.process * 1.8 + 270) * 16,
        #                   self.process * 3.6 * 16)  # blue upper drawChord(x,y, 起始角度，跨越角度）度数要*16.
        # painter.drawChord(QtCore.QRect(self.upBlueXPoint, self.lowBlueYPoint, self.radius * 2,
        #                                self.radius * 2), (-self.process * 1.8 + 270) * 16,
        #                   self.process * 3.6 * 16)  # blue lower

        # 画弧
        painter.setBrush(self.water_color)
        painter.drawChord(2,2,self.w-4,self.h-4,
                          int((-self.cu_value*1.8+270)*16),
                          int(self.cu_value*self.en*16))

    # 绘制文字
    def drawText_(self, painter: QPainter):
        if not self.__is_hide_text:
            # 绘制文字
            f = QFont()
            f.setBold(True)
            f.setPointSize(self.text_size)
            painter.setFont(f)
            painter.setPen(self.text_color)
            # 文字
            fs = QFontMetricsF(f)
            fw = int(fs.width(self.text))
            fh = int(fs.height())
            painter.drawText(self.w // 2 - fw // 2, self.h // 2 + fh // 4, self.text)

    def paintEvent(self, e: QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)

        # 画水位
        self.drawWaterLevel(painter)
        # 画文字
        self.drawText_(painter)

        painter.end()

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        super().resize(self.w,self.h)

    def resizeEvent(self, e: QResizeEvent) -> None:
        self.w = e.size().width()
        self.h = e.size().height()
        super().resizeEvent(e)



# -*- coding:utf-8 -*-
# @time:2022/12/259:42
# @author:LX
# @file:test_SlideShow.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QLabel,
    Qt,
    QHBoxLayout,
)

from PyQtGuiLib.core import SlideShow


class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(1000,600)

        self.flag = True

        # 窗口轮播功能
        self.ss = SlideShow(self)
        # self.ss.setAinDirectionMode((SlideShow.Ani_Up,SlideShow.Ani_Down))
        self.ss.setAutoSlideShow(True,direction=SlideShow.Ani_Down,is_not_show_btn=False)
        # self.ss.setHideButtons(True)
        self.ss.move(50,50)
        self.ss.resize(600,200)
        self.test()  # 添加测试窗口

        self.ss.switchWidgeted.connect(self.my_event)

    def my_event(self,index:int):
        print("当前索引:",index)
        # if self.flag:
        #     self.ss.removeWidget(2)
        #     self.flag = False

    def test(self):
        t = QWidget()
        tl = QLabel("1")
        tl.setAlignment(Qt.AlignCenter)
        lh = QHBoxLayout(t)
        lh.addWidget(tl)
        tl.setStyleSheet('font: 60pt "微软雅黑";')
        t.setStyleSheet("background-color:#00aa7f;")
        t2 = QWidget()
        tl2 = QLabel("2")
        tl2.setAlignment(Qt.AlignCenter)
        lh2 = QHBoxLayout(t2)
        lh2.addWidget(tl2)
        tl2.setStyleSheet('font: 60pt "微软雅黑";')
        t2.setStyleSheet("background-color:#aaaaff;")
        t3 = QWidget()
        tl3 = QLabel("3", t3)
        tl3.setAlignment(Qt.AlignCenter)
        lh3 = QHBoxLayout(t3)
        lh3.addWidget(tl3)
        tl3.setStyleSheet('font: 60pt "微软雅黑";')
        t3.setStyleSheet("background-color:green;")
        self.ss.addWidget(t)
        self.ss.addWidget(t2)
        self.ss.addWidget(t3)



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/109:49
# @author:LX
# @file:bubbleWidget.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QWidget,
    QPainter,
    QPainterPath,
    QPaintEvent,
    QPolygonF,
    QRectF,
    QFont,
    QColor,
    QPointF,
    Signal,
    QThread,
    QSize,
    QFontMetricsF
)

from PyQtGuiLib.animation.lmlmAni import LmLmAnimation
'''
    气泡窗口
    种类一
          /\
    ------  -------
    |             |
    ---------------
    # 后续更新动画
'''

# 弹窗持续时间
class DurationTimeThread(QThread):
    finished = Signal()  # 完成信号
    def __init__(self,bub,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.bub = bub  # type:BubbleWidget
        self.dur_time = 3  # 默认秒

    def setDurationTime(self,seconds:int):
        self.dur_time = seconds

    def run(self) -> None:
        if self.dur_time == -1:
            self.finished.emit()
            return

        n = 0
        while n != self.dur_time:
            self.sleep(1)
            n+=1
        self.finished.emit()


class BubbleWidget(QWidget):
    # 启动/结束事件
    finished = Signal()

    Top = "top"
    Down = "Down"
    Left = "Left"
    Right = "Right"
    NoNone = "None"

    # 弹窗永远存在,不好消失
    Be_Forever = -1

    def __init__(self,*args,**kwargs):
        self.triangle_km = 10  # 三角形的开口大小
        self.w, self.h = 160, 70  # 默认大小
        super().__init__(*args,**kwargs)
        self.resize(self.w,self.h)
        self.setObjectName("BubbleW")

        self.box = 2  # 边距
        self.triangle_pos = 20  # 三角形在矩形x/y轴的什么位置
        self.triangle_diameter = 15 # 三角形的高度
        self.direction = BubbleWidget.Top  # 三级形的方向(默认三角形在上面)
        self.radius = 5 # 半径
        self.bcolor = QColor(152, 167, 255)  # 气泡颜色
        self.text = "Bubble"  # 文本
        self.text_color = QColor(0,85,0)  # 文字颜色
        self.text_size = 16  # 文字大小
        self.is_aim = False # 判断是否需要动画(动画的优先级姚高于持续时间 先执行动画->在执行气泡持续时间)
        self.is_durtime = False # 是否开启气泡窗口的持续时间
        # ===
        self.dtthread = DurationTimeThread(self)  # 气泡持续时间线程
        self.lmlm = LmLmAnimation()  # 动画
        self.lmlm.setTargetObject(self) # 设置动画的目标
        #
        self.dtthread.finished.connect(self.finish_event)

    # 气泡启动动画
    def setAnimationEnabled(self,b:bool,duration_time:int=2000):
        if b:
            self.is_aim = True
            self.lmlm.setDuration(duration_time)
            self.lmlm.start()
            if self.is_durtime:
                def _(self):
                    self.dtthread.start()
                    self.lmlm.disconnect()  # 这里必须断开链接,
                self.lmlm.finished.connect(lambda :_(self))

        else:
            self.is_aim = False

    # 气泡持续时间
    def setDurationTime(self, seconds: int):
        if seconds != BubbleWidget.Be_Forever:
            self.is_durtime = True
            self.dtthread.setDurationTime(seconds)
            if self.is_aim is False:
                self.dtthread.start()
        else:
            self.is_durtime = False

    # 完成事件
    def finish_event(self):
        if self.is_durtime and self.is_aim is False:
            self.close()
        else:
            self.lmlm.setMode(self.lmlm.Hide)
            self.lmlm.start()
            # self.lmlm.finished.connect(self.finished.emit())

    # 追踪控件
    def setTrack(self,widget:QWidget,offset:int=0):
        '''
            widget:需要跟踪的控件
            offset:偏移距离(默认居中)
        '''
        w, h = widget.width(), widget.height()
        x, y = widget.x(), widget.y()
        xw, hy = x + w, h + y
        # 轴上的顶点居中位置
        center_vertex_x = w//2-self.triangle_pos-self.triangle_km+offset
        center_vertex_y = h//2-self.triangle_pos-self.triangle_km+offset
        if self.direction == BubbleWidget.Top:
            self.move(x+center_vertex_x,y+h)
        elif self.direction == BubbleWidget.Down:
            self.move(x+center_vertex_x,y-self.h)
        elif self.direction == BubbleWidget.Right:
            self.move(x-self.w,y+center_vertex_y)
        elif self.direction == BubbleWidget.Left:
            self.move(xw,y+center_vertex_y)

    def resize(self,*args) -> None:
        '''
            兼容qt原本的实现
        '''
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w,self.h = args[0].width(),args[0].height()
        else:
            self.w,self.h =args
        super().resize(*args)

    def setText(self,text:str):
        self.text = text

    def setTextColor(self,color:QColor):
        self.text_color = color

    def setTextSize(self,size:int):
        self.text_size = size

    def setAllText(self,text:str,size=None,color:QColor=None):
        self.setText(text)
        if size:
            self.setTextSize(size)
        if color:
            self.setTextColor(color)

    def setKmPos(self,pos:int):
        self.triangle_pos = pos

    def setKmDiameter(self,diameter:int):
        self.triangle_diameter = diameter

    def setKmM(self,km:int):
        self.triangle_km = km

    def setKm(self,pos:int,diameter:int,km:int):
        '''
            pos:三角形在矩形的距离
            diameter:三角形的高度
            km:三角形开口大小
        '''
        self.setKmPos(pos)
        self.setKmDiameter(diameter)
        self.setKmM(km)

    # 设置方向
    def setDirection(self,d):
        self.direction = d

    def setBColor(self,bcolor:QColor):
        self.bcolor = bcolor

    # 三角形
    def delta(self,ppath:QPainterPath):
        if self.direction == BubbleWidget.Top:
            ploys = [QPointF(self.triangle_pos,self.triangle_diameter+self.box),
                     QPointF(self.triangle_pos+self.triangle_km,self.box),
                     QPointF(self.triangle_pos+self.triangle_km<<1,self.triangle_diameter+self.box)]
        elif self.direction == BubbleWidget.Down:
            ploys = [QPointF(self.triangle_pos,self.h-self.triangle_diameter),
                     QPointF(self.triangle_pos+self.triangle_km,self.h-self.box),
                     QPointF(self.triangle_pos+self.triangle_km<<1,self.h-self.triangle_diameter)]
        elif self.direction == BubbleWidget.Left:
            ploys = [QPointF(self.triangle_diameter+self.box,self.triangle_pos),
                     QPointF(self.box,self.triangle_pos+self.triangle_km),
                     QPointF(self.triangle_diameter+self.box,self.triangle_pos+self.triangle_km<<1)]
        elif self.direction == BubbleWidget.Right:
            ploys =[QPointF(self.w-self.triangle_diameter,self.triangle_pos),
                   QPointF(self.w-self.box,self.triangle_pos+self.triangle_km),
                   QPointF(self.w-self.triangle_diameter,self.triangle_pos+self.triangle_km<<1)]
        else:
            return
        ppath.addPolygon(QPolygonF(ploys))

    # 矩形
    def ract_(self,ppath:QPainterPath):
        if self.direction == BubbleWidget.Top:
            rectf = QRectF(self.box, self.box + self.triangle_diameter,
                           self.w - self.box, self.h - self.triangle_diameter - self.box)
        elif self.direction == BubbleWidget.Down:
            rectf = QRectF(self.box, self.box,
                           self.w - self.box, self.h - self.triangle_diameter - self.box)
        elif self.direction == BubbleWidget.Left:
            # self.triangle_diameter+self.box 三角占用的空间
            rectf = QRectF(self.triangle_diameter+self.box,self.box,
                           self.w-self.triangle_diameter,self.h-self.box)
        elif self.direction == BubbleWidget.Right:
            rectf = QRectF(self.box,self.box,
                           self.w-self.triangle_diameter-self.box,
                           self.h-self.box)
        else:
            rectf = QRectF(self.box, self.box,
                           self.w - self.box, self.h - self.box)
        ppath.addRoundedRect(rectf, self.radius,self.radius)

    # 文字
    def text_(self,painter:QPainter):
        f = QFont()
        f.setPointSize(self.text_size)
        painter.setFont(f)
        painter.setPen(self.text_color)
        # 文字
        fs = QFontMetricsF(f)
        fw = int(fs.width(self.text))
        fh = int(fs.height())
        if self.direction == BubbleWidget.Top:
            x = (self.w - fw) // 2
            y = self.h // 2+self.triangle_diameter
        elif self.direction == BubbleWidget.Down:
            x = (self.w - fw) // 2
            y = (self.h-self.triangle_diameter+fh) // 2
        elif self.direction == BubbleWidget.Left:
            x = (self.w-self.triangle_diameter-fw//2)//2
            y = (self.h + fh)//2
        elif self.direction == BubbleWidget.Right:
            x = (self.w-self.triangle_diameter)//2-fw//2
            y = (self.h + fh) // 2
        else:
            x,y = 0,0
            self.text = ""
        painter.drawText(x,y,self.text)

    def paintEvent(self, e:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        # -------------------
        ppath = QPainterPath()
        # 绘制矩形
        self.ract_(ppath)
        # 绘制三角形
        self.delta(ppath)
        # -------------------
        painter.fillPath(ppath,self.bcolor)
        # 绘制文字
        self.text_(painter)
        painter.end()






from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QMainWindow
)
from PyQtGuiLib.core.widgets import ButtonWidget
from PyQtGuiLib.core import PullOver


class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(500,500)

        self.show_btn = ButtonWidget()
        self.show_btn.resize(70, 70)
        self.show_btn.setObjectName("show_btn")
        self.show_btn.setStyleSheet('''
        #show_btn{
        background-color:green;
        border-radius:35px;
        }
        ''')

        # 窗口靠边功能
        self.pullOver = PullOver(self)
        # self.pullOver.setEasingCurve(PullOver.OutBounce)
        self.pullOver.pullover(self.show_btn)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2022/12/2912:42
# @author:LX
# @file:comboBox2.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QThread,
    Signal,
    QLineEdit,
    QInputMethodEvent,
    QKeyEvent,
    QPaintEvent,
    QMouseEvent,
    QPainter,
    QPen,
    QFont,
    QResizeEvent,
    QRect,
    QBrush,
    QColor,
    QScrollArea,
    QVBoxLayout,
    QPushButton,
    QLabel,
    Qt,
    QPropertyAnimation,
    QPoint,
    QSize,
    QMoveEvent,
    QComboBox
)

class ScrollArea(QScrollArea):
    # 风格
    Style_Card = "card"  # 卡片

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.resize(500,500)

        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # 风格模式
        self.style_mode = ScrollArea.Style_Card

        self.__vlay = QVBoxLayout(self)
        self.__vlay.setContentsMargins(0,0,0,0)
        self.__vlay.setSpacing(3)

        self.item_fixed_size = (self.width(),30)

    # 卡片风格颜色
    def styleCrad(self)->str:
        style ='''
background-color: rgb(176, 255, 225);
border:1px solid gray;
border-radius:5px;
        '''
        return style

    def addWidget(self,widget:QWidget):
        if isinstance(widget,str):
            widget = QLabel(widget)
            print(widget)
            widget.setFixedHeight(30)
            widget.setAlignment(Qt.AlignCenter)
            if self.style_mode == ScrollArea.Style_Card:
                widget.setStyleSheet(self.styleCrad())
        self.__vlay.addWidget(widget)
        self.adjustSize()


class LineEdit(QLineEdit):
    clicked = Signal() # 点击事件

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.resize(300,40)
        # 图标大小
        self.icon_w, self.icon_h = 12, 12
        # 图标位置靠右的位置,距离边的距离
        self.icon_right_margin = 25
        # 圆角
        self.icon_radius = (6,6)
        self.icon_color = QColor(0, 255, 0)

        # 动画开关
        self.animation_switch = False

        #
        self.scroll_area = ScrollArea()
        # self.startTimer(100)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.clicked.emit()
        self.animation_switch = True
        self.scroll_area.show()
        self.scroll_area.move(self.pos().x(),self.pos().y()+70)
        self.scroll_area.resize(self.width(),200)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e: QMouseEvent) -> None:
        super().mouseReleaseEvent(e)

    def moveEvent(self, e: QMoveEvent) -> None:
        self.scroll_area.move(e.pos().x(),e.pos().y()+self.height())
        super().moveEvent(e)

    # 绘制图标
    def drawIcon(self,painter:QPainter):
        parent = self # 父类对象
        h = self.height()//2-self.icon_h//2
        rect = QRect(parent.width() - self.icon_right_margin, h, self.icon_w,self.icon_h)

        op = QPen()
        op.setColor(self.icon_color)
        bush = QBrush(self.icon_color)

        painter.setPen(op)
        painter.setBrush(bush)

        painter.drawRoundedRect(rect,*self.icon_radius)

    # def timerEvent(self, e) -> None:
    #     self.update()

    def paintEvent(self, e: QPaintEvent) -> None:
        super().paintEvent(e)
        painter = QPainter(self)
        painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)

        # 绘制图标
        self.drawIcon(painter)

        painter.end()


    def closeEvent(self, e) -> None:
        self.scroll_area.close()
        super().closeEvent(e)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = LineEdit()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/128:53
# @author:LX
# @file:developmentLog.py
# @software:PyCharm
'''
    开发日志文件
'''


'''
2022.12.13  0.0.1.4版本  气泡窗口已经基本实现,持续优化更新
2022.12.13  0.0.1.5版本  重写了气泡窗口的部分实现代码
2022.12.19  0.0.2.0版本
    --- 气泡窗口 优化了代码,修复了文字大小无法设置的BUG,以及文字位置的问题
    --- 新增无边框窗口 BorderlessWidget 类,导入方式 from PyQtGuiLib.core.widgets import BorderlessWidget
    --- 新增圆角窗口 RoundWidget 类,导入方式 from PyQtGuiLib.core.widgets import RoundWidget
    --- 靠边窗口功能完成(任意窗口都可以加上该功能),PullOver类 导入方式 from PyQtGuiLib.core.pullOver import PullOver
    --- 靠边窗口功能的测试用例已编写
2022.12.20  0.0.3.0版本
    --- 新增圆环进度条
2022.12.22  0.0.4.0版本
    --- 修复了一些小的BUG
    --- 放弃了亚克力窗口的研究
    --- 新增加载进度条
2022.12.22  0.0.5.0版本
    --- 新增水球进度条
    --- 新增进度条的集合测试
    --- 修改了气泡窗口的导入方式
         原来的导入方式: from PyQtGuiLib.core.bubbleWidget import BubbleWidget
         现在的导入方式: from PyQtGuiLib.core import BubbleWidget
    --- 修改了窗口靠边功能的导入方式
         原来的导入方式: from PyQtGuiLib.core.pullOver import PullOver
         现在的导入方式: from PyQtGuiLib.core import PullOver
2022.12.26  0.0.6.0版本
    --- 新增轮播组件,2022.12.22  已经基本完成,可能还存在BUG,以后修改(测试发布)
2022.12.26  0.0.6.1版本
    --- 新增轮播组件 新增两个方法,setAnimationTime(),setAinDirectionMode()
2022.12.26  0.0.7.0版本
    --- 新增线性渐变进度条,到目前为止,这是唯一个不提供文字显示的进度条
    --- 更新的所有进度条的抗锯齿写法 painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)
2023.1.4  0.0.8.0版本
    --- 线性渐变进度条 增加一个修改背景底色的方法 setBackGroundColor()
    --- 感谢[PyQt5学习爱好群-(讨厌自己)提供的BUG修复思路] 修复Borderless右下角拉伸BUG
    --- 新增 标题栏 TitleBar 类(测试发布中)
2023.1.4 0.0.8.1版本
    --- 修复 TitleBar 标题栏 在多个屏幕下的BUG
    --- TitleBar 在点击关闭按钮,新增隐退效果
    --- TitleBar 放大按钮动画增强
    --- TitleBar 增加缩小动画
2023.1.4 0.0.8.2版本
    --- TitleBar 更新Mac风格的放大,缩小,关闭按钮风格
    --- TitleBar 新增 setAniDuration() 方法 设置动画的时长
    --- TitleBar 新增 setTitleIcon(),setSyncWindowIcon() 与设置图标相关的方法
    --- 优化了TitleBar部分代码
2023.1.4 0.0.8.3版本
    --- 修复 TitleBar 类在mac下图标显示位置的问题
    --- utility.py 新增 系统判断
    --- 新增状态栏
'''


# -*- coding:utf-8 -*-
# @time:2022/12/2317:54
# @author:LX
# @file:SlideShow.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QResizeEvent,
    QPushButton,
    QPoint,
    QSize,
    QPropertyAnimation,
    Signal
)



'''

    轮播图功能
        风格: 扁平
'''
class SlideShow(QWidget):
    # 切换窗口事件
    switchWidgeted = Signal(int)

    # 扁平 风格
    FlatMode = "flat"  # 经典

    # 动画的方向模式
    Ani_Left = "left"
    Ani_Right = "right"
    Ani_Down = "down"
    Ani_Up = "up"


    # 动画
    Translation = "translation"  # 平移

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(700, 300)

        self.show_mode = SlideShow.FlatMode # 展示模式
        self.animation_mode = SlideShow.Translation  # 动画模式

        self.animation_time = 300 # 毫秒

        self.index = [-1,0]  # 索引
        self.cu_index = self.index[1]  # 当前索引

        # 设置自动轮播,自动轮播的方向
        self.is_auto_slide = False
        self.auto_direction = SlideShow.Ani_Right

        # 左右按钮
        self.Ani_Left_btn = QPushButton("左",self)
        self.Ani_Right_btn = QPushButton("右",self)
        self.Ani_Left_btn.setObjectName("Ani_Left_btn")
        self.Ani_Right_btn.setObjectName("Ani_Right_btn")
        self.Ani_Left_btn.resize(50,50)
        self.Ani_Right_btn.resize(50,50)
        self.Ani_Left_btn.clicked.connect(lambda :self.roll_event(SlideShow.Ani_Left))
        self.Ani_Right_btn.clicked.connect(lambda :self.roll_event(SlideShow.Ani_Right))
        self.btnStyle()

        # 窗口
        self.widgets = []  # 所有已经添加的窗口
        self.current_widgets = None # 当前已经展示出来的轮播窗口对象

        self.defaultBackground()

    def defaultBackground(self):
        '''
            默认背景,只有当前没有添加任何窗口时显示
        :return:
        '''
        print(self.isEmpty())
        if self.isEmpty():
            self.setStyleSheet('''
background-color:gray;
border-radius:5px;
            ''')
    
    # 设置动画方向模式
    def setAinDirectionMode(self,modes:tuple):
        if isinstance(modes,tuple):
            self.Ani_Left_btn.disconnect()
            self.Ani_Right_btn.disconnect()
            self.Ani_Left_btn.clicked.connect(lambda: self.roll_event(modes[0]))
            self.Ani_Right_btn.clicked.connect(lambda: self.roll_event(modes[1]))

    # 设置动画间隔时间
    def setAnimationTime(self,interval:int=300):
        self.animation_time = interval

    # 自动轮播
    def setAutoSlideShow(self,b:bool,interval:int=2000,direction: str="Ani_Right",is_not_show_btn:bool=True):
        '''
            b: 是否启动自动轮播
            interval:切换窗口的事件间隔
            direction:轮播方向
            is_not_show_btn:是否需要显示左右按钮
        '''
        self.is_auto_slide = b
        if b:
            self.startTimer(interval)

        self.setHideButtons(is_not_show_btn)

        self.auto_direction = direction
        self.Ani_Left_btn.setEnabled(not b)
        self.Ani_Right_btn.setEnabled(not b)

    # 设置隐藏/显示左右按钮
    def setHideButtons(self,b:bool=False):
        if b:
            self.Ani_Left_btn.hide()
            self.Ani_Right_btn.hide()
        else:
            self.Ani_Left_btn.show()
            self.Ani_Right_btn.show()

    # 获取当前窗口的数量
    def getWidgetCount(self) -> int:
        return len(self.widgets)

    # 轮播事件
    def roll_event(self, direction: str):
        if self.show_mode == SlideShow.FlatMode:  # 经典模式
            widgets_count = self.getWidgetCount()

            if self.isEmpty():
                return

            self.index[0] += 1
            self.index[1] += 1

            if self.index[0] > widgets_count - 1:
                self.index[0] = 0

            if self.index[1] > widgets_count - 1:
                self.index[1] = 0

            self.cu_index = self.index[1] # 设置当前索引
            self.switchWidgeted.emit(self.index[1]) # 发送事件
            self.rollShow(direction)

    # 通过索引获取窗口
    def getWidget(self,index:int) -> QWidget:
        return self.widgets[index]

    def isEmpty(self) -> bool:
        return True if not self.widgets else False

    # 滚动显示
    def rollShow(self,direction: str):
        widget = self.widgets[self.index[0]] # type:QWidget
        widget2 = self.widgets[self.index[1]] # type:QWidget
        self.__createWidget(widget2)
        self.animation_(direction,widget,widget2)

    # 添加窗口
    def addWidget(self,widget:QWidget):
        self.widgets.append(widget)
        # 展示
        self.show_()

    # 移除窗口(2022.12.26该方法处于测试中,可能有BUG)
    def removeWidget(self,obj_or_index):
        '''
            obj_or_index: 窗口对象,或者是索引
        '''

        if isinstance(obj_or_index,int):
            widget = self.widgets[obj_or_index] # type:QWidget
            self.widgets.remove(widget)
            widget.hide()
        else:
            obj_or_index.hide()
            self.widgets.remove(obj_or_index)

    # 切换到指定窗口(暂时放弃改函数)
    def setCurrentIndex(self,index:int=0,is_animation:bool=False):
        if not is_animation:
            widget = self.getWidget(index)

            self.__createWidget(widget)

    # 获取当前索引
    def getIndex(self)->int:
        if self.isEmpty():
           return None

        return self.cu_index

    # 切换到下一个窗口
    def next(self):
        self.roll_event(SlideShow.Ani_Right)

    # 切换到上一个窗口
    def Ani_Up(self):
        self.roll_event(SlideShow.Ani_Left)

    # 两侧的按钮
    def btnPos(self):
        self.Ani_Left_btn.move(5, self.height() // 2 - self.Ani_Left_btn.height() // 2)
        self.Ani_Right_btn.move(self.width() - self.Ani_Right_btn.width() - 5,
                            self.height() // 2 - self.Ani_Left_btn.height() // 2)

    # 按钮样式
    def btnStyle(self):
        self.Ani_Left_btn.setStyleSheet('''
#Ani_Left_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Left_btn:hover{
border:2px solid #00557f;
}
        ''')
        self.Ani_Right_btn.setStyleSheet('''
#Ani_Right_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Right_btn:hover{
border:2px solid #00557f;
}
                ''')

    # 返回左右按钮对象
    def getButtons(self) -> tuple:
        return self.Ani_Left_btn,self.Ani_Right_btn

    # 展示
    def show_(self):
        if self.show_mode == SlideShow.FlatMode:  # 经典模式

            if self.isEmpty():
                return

            # 优先展示第一个窗口
            widget = self.widgets[self.cu_index]  # type:QWidget
            self.__createWidget(widget)

            # 隐藏除了显示的以外的所有窗口
            for i in range(self.getWidgetCount()):
                if i != self.getIndex():
                    self.getWidget(i).hide()

    # 创建窗口
    def __createWidget(self,widget:QWidget):
        widget.setParent(self)
        widget.resize(QSize(self.width(),self.height()))
        widget.lower()
        widget.move(QPoint(0,0))
        widget.show()

    # 动画
    def animation_(self,direction:str,widget:QWidget,widget2:QWidget):
        if self.animation_mode == SlideShow.Translation:
            if len(self.widgets) >= 2:
                move_ani = QPropertyAnimation(self)
                move_ani2 = QPropertyAnimation(self)
                move_ani.setPropertyName(b"pos")
                move_ani2.setPropertyName(b"pos")
                move_ani.setTargetObject(widget)
                move_ani2.setTargetObject(widget2)

                move_ani.setStartValue(widget.pos())
                if direction == SlideShow.Ani_Right:
                    end_value,start_value = QPoint(widget.width(), 0),QPoint(-widget.width(), 0)
                elif direction == SlideShow.Ani_Left:
                    end_value,start_value = QPoint(-widget.width(), 0),QPoint(widget.width(), 0)
                elif direction == SlideShow.Ani_Down:
                    end_value,start_value = QPoint(0,widget.height()),QPoint(0,-widget.height())
                elif direction == SlideShow.Ani_Up:
                    end_value, start_value = QPoint(0, -widget.height()), QPoint(0, widget.height())
                else:
                    end_value, start_value = QPoint(widget.width(), 0), QPoint(-widget.width(), 0)
                move_ani.setEndValue(end_value)
                move_ani2.setStartValue(start_value)
                move_ani2.setEndValue(QPoint(0, 0))


                move_ani.setDuration(self.animation_time)
                move_ani2.setDuration(self.animation_time)

                move_ani.start()
                move_ani2.start()

    def resizeEvent(self, e:QResizeEvent) -> None:
        self.btnPos()
        self.show_()
        super().resizeEvent(e)

    def timerEvent(self, e) -> None:
        if self.is_auto_slide:
            self.roll_event(self.auto_direction)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = SlideShow()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/1018:13
# @author:LX
# @file:test_BubbleWidget.py
# @software:PyCharm

from PyQtGuiLib.header import PYQT_VERSIONS
from PyQtGuiLib.header import (
    sys,
    QApplication,
    QWidget,
    QPushButton,
)

from PyQtGuiLib.core import BubbleWidget

'''
    气泡窗口的测试用例
'''

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.setWindowOpacity(0.5)


        self.btn = QPushButton("一号玩家",self)
        self.btn.resize(130,100)
        self.btn.move(200,200)

        # 气泡控件
        self.bu = BubbleWidget(self)
        self.bu.resize(150,100)
        self.bu.setDirection(BubbleWidget.Top)
        self.bu.setDurationTime(BubbleWidget.Be_Forever)
        self.bu.setAnimationEnabled(True)
        self.bu.setTrack(self.btn)
        self.bu.setText("你好世界")
        # self.bu.setText("hello world")

        self.bu.finished.connect(lambda :print("111111111"))
        # self.bu.move(80,100) # 如果不想手动设置位置可以用下面控件追踪功能

        # # --------
        # self.btn2 = QPushButton("二号玩家",self)
        # self.btn2.resize(150,80)
        # self.btn2.move(400,350)
        #
        # self.bu2 = BubbleWidget(self)
        # self.bu2.setText("你才是哼")
        # # 反向的设置一定要在追踪前面
        # self.bu2.setDurationTime(BubbleWidget.Be_Forever)
        # self.bu2.setDirection(BubbleWidget.Left)
        # self.bu2.setTrack(self.btn2)
        # self.bu2.setTextColor(QColor(255, 255, 0))
        # self.bu2.setBColor(QColor(170, 0, 255))
        # # self.bu2.setKm(30,10)
        # self.bu2.resize(260,80)
        #
        # self.bu3 = BubbleWidget(self)
        # self.bu3.setText("绿色")
        # self.bu3.setDirection(BubbleWidget.NoNone)
        # self.bu3.setBColor(QColor(170, 255, 127))
        # self.bu3.move(80,300)
        # self.bu3.setKm(80,25)
        # self.bu3.resize(160,80)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    Signal,
    Qt,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent
)


'''
    节点进度条(暂时停止研究)
'''
class NodeBar(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.w,self.h =100, 50
        self.resize(600,50)

    def drawEllipses(self,painter:QPainter,n):
        x = 100
        xx=100
        for i in range(n):
            painter.drawEllipse(x, 0, self.h, self.h)
            x+=self.h+xx

    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)

        op = QPen()
        op.setColor(QColor(179, 179, 179))
        painter.setPen(op)
        painter.setBrush(QColor(179, 179, 179))

        painter.drawRoundedRect(2,self.height()//2-self.h//4,self.width()-4,self.h//2,5,5)
        self.drawEllipses(painter,3)
        painter.end()

    def resizeEvent(self, event:QResizeEvent) -> None:
        self.h =event.size().height()
        super().resizeEvent(event)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = NodeBar()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from ctypes import POINTER, c_bool, sizeof, windll, pointer, c_int
from ctypes.wintypes import DWORD

from win32 import win32api, win32gui
from win32.lib import win32con

from cstruct import ACCENT_POLICY, ACCENT_STATE, WINDOWCOMPOSITIONATTRIB, WINDOWCOMPOSITIONATTRIBDATA


class WindowEffect:
    def __init__(self):
        self.SetWindowCompositionAttribute = windll.user32.SetWindowCompositionAttribute
        self.SetWindowCompositionAttribute.restype = c_bool
        self.SetWindowCompositionAttribute.argtypes = [
            c_int, POINTER(WINDOWCOMPOSITIONATTRIBDATA)]

        self.accentPolicy = ACCENT_POLICY()
        self.winCompAttrData = WINDOWCOMPOSITIONATTRIBDATA()
        self.winCompAttrData.Attribute = WINDOWCOMPOSITIONATTRIB.WCA_ACCENT_POLICY.value[0]
        self.winCompAttrData.SizeOfData = sizeof(self.accentPolicy)
        self.winCompAttrData.Data = pointer(self.accentPolicy)

    def setAcrylicEffect(self, hWnd: int, gradientColor: str = 'F0000000', isEnableShadow: bool = True,
                         animationId: int = 0):

        gradientColor = gradientColor[6:] + gradientColor[4:6] + \
                        gradientColor[2:4] + gradientColor[:2]
        gradientColor = DWORD(int(gradientColor, base=16))
        animationId = DWORD(animationId)
        accentFlags = DWORD(0x20 | 0x40 | 0x80 | 0x100) if isEnableShadow else DWORD(0)

        self.accentPolicy.AccentState = ACCENT_STATE.ACCENT_ENABLE_ACRYLICBLURBEHIND.value[0]
        self.accentPolicy.GradientColor = gradientColor
        self.accentPolicy.AccentFlags = accentFlags
        self.accentPolicy.AnimationId = animationId
        self.SetWindowCompositionAttribute(hWnd, pointer(self.winCompAttrData))

    def setAeroEffect(self, hWnd: int):
        self.accentPolicy.AccentState = ACCENT_STATE.ACCENT_ENABLE_BLURBEHIND.value[0]
        self.SetWindowCompositionAttribute(hWnd, pointer(self.winCompAttrData))

    @staticmethod
    def moveWindow(hWnd: int):
        win32gui.ReleaseCapture()
        win32api.SendMessage(hWnd, win32con.WM_SYSCOMMAND,
                             win32con.SC_MOVE + win32con.HTCAPTION, 0)


# -*- coding:utf-8 -*-
# @time:2022/12/2317:54
# @author:LX
# @file:SlideShow.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QResizeEvent,
    QPushButton,
    QStackedWidget,
    QMouseEvent,
    QPoint,
    QSize
)



'''
    弃案
    轮播图功能
        风格: 扁平,卡片
'''
class SlideShow(QWidget):
    # 扁平/卡片 风格
    FlatMode = "flat"  # 经典
    CardMode = "card"  # 卡片

    # 方向
    Left = "left"
    Right = "right"

    # 动画
    Translation = "translation"  # 平移
    FadeOut = "fadeout" # 渐隐渐显

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(700, 300)

        self.show_mode = SlideShow.FlatMode # 展示模式
        self.animation_mode = SlideShow.Translation  # 动画模式

        self.index = 0  # 索引

        # 左右按钮
        self.left_btn = QPushButton("左",self)
        self.right_btn = QPushButton("右",self)
        self.left_btn.setObjectName("left_btn")
        self.right_btn.setObjectName("right_btn")
        self.left_btn.resize(50,50)
        self.right_btn.resize(50,50)
        self.left_btn.clicked.connect(lambda :self.roll_event(SlideShow.Left))
        self.right_btn.clicked.connect(lambda :self.roll_event(SlideShow.Right))

        # 窗口
        self.widgets = []  # 所有已经添加的窗口
        self.current_widgets = [] # 当前已经展示出来的轮播窗口对象

        # 测试
        self.test()

    def test(self):
        t = QWidget()
        t.setStyleSheet("background-color:red;")
        t2 = QWidget()
        t2.setStyleSheet("background-color:blue;")
        t3 = QWidget()
        t3.setStyleSheet("background-color:green;")
        self.addWidget(t)
        self.addWidget(t2)
        self.addWidget(t3)

    # 返回当前窗口上已经展示的轮播对象窗口
    def currentWidgets(self) -> list:
        return self.current_widgets

    # 隐藏当前已经展示的轮播对象窗口
    def hideCurrentWidgets(self):
        for widget in self.currentWidgets():
            widget.hide()

    # 轮播事件
    def roll_event(self, direction: str):
        if self.show_mode == SlideShow.FlatMode:  # 经典模式
            widgets_count = len(self.widgets)

            if direction == SlideShow.Left:
                self.index -= 1
                if self.index < 0:
                    self.index = widgets_count - 1
            if direction == SlideShow.Right:
                self.index += 1
                if self.index > widgets_count - 1:
                    self.index = 0
            self.hideCurrentWidgets()
            self.show_()

        if self.show_mode == SlideShow.CardMode:  # 卡片模式
            widgets_count = len(self.widgets)

            if widgets_count >= 2:
                if direction == SlideShow.Left:
                    pass
                if direction == SlideShow.Right:
                    pass

    def addWidget(self,widget:QWidget):
        self.widgets.append(widget)
        # 展示
        self.show_()

    # 两侧的按钮
    def btnPos(self):
        self.left_btn.move(5, self.height() // 2 - self.left_btn.height() // 2)
        self.right_btn.move(self.width() - self.right_btn.width() - 5,
                            self.height() // 2 - self.left_btn.height() // 2)

    # 展示
    def show_(self):
        # 隐藏清除
        self.hideCurrentWidgets()
        self.current_widgets.clear()

        if self.show_mode == SlideShow.FlatMode:  # 经典模式
            # 优先展示第一个窗口
            widget = self.widgets[self.index]  # type:QWidget
            widget.setParent(self)
            widget.show()
            widget.lower()
            widget.resize(self.width(),self.height())

            # 添加到当前展示的窗口
            self.current_widgets.append(widget)

        # -----------------------------------------------
        if self.show_mode == SlideShow.CardMode:  # 卡片模式
            '''
                展示窗口的时候
                    如果只有一个窗口,那直接居中展示
                    两个窗口时,一个左边,一个居中
                    三个窗口时,一个左边一个居中一个右边
                    更多窗口时,优先展示前三个
            '''
            widgets_count = len(self.widgets)
            if not widgets_count:
                return

            # 每个轮播窗口的大小
            size = QSize(self.width() // 2, self.height() // 2)

            if widgets_count == 1:
                widget = self.widgets[0] # type:QWidget
                self.current_widgets.append(widget)
                # 居中
                center_pos = QPoint(self.width() // 2 - widget.width() // 2,
                                    self.height() // 2 - widget.height() // 2)
                self.__createWidget(widget,size,center_pos)
            elif widgets_count == 2:
                left_widget,widget = self.widgets
                self.current_widgets.extend(self.widgets)

                # 居中
                center_pos = QPoint(self.width() // 2 - widget.width() // 2,
                                    self.height() // 2 - widget.height() // 2)
                # 左边轮播图的位置
                left_pos = QPoint(center_pos.x() - self.width()//5,
                                  center_pos.y())

                left_size = QSize(self.width()//4,self.height()//2)
                center_size = QSize(self.width()//3,self.height()//2)
                self.__createWidget(left_widget, left_size, left_pos)
                self.__createWidget(widget, center_size, center_pos)
            elif widgets_count >= 3:
                left_widget, widget,right_widget = self.widgets[:3]
                self.current_widgets.extend(self.widgets[:3])

                # 居中
                center_pos = QPoint(self.width() // 2 - widget.width() // 2,
                                    self.height() // 2 - widget.height() // 2)
                # 左边轮播图的位置
                left_pos = QPoint(center_pos.x() - self.width() // 5,
                                  center_pos.y())

                # 右边轮播图的位置
                right_pos = QPoint(center_pos.x() + self.width() // 4,
                                  center_pos.y())

                left_size = QSize(self.width() // 4, self.height() // 2)
                right_size = left_size
                center_size = QSize(self.width() // 3, self.height() // 2)
                self.__createWidget(left_widget, left_size, left_pos)
                self.__createWidget(right_widget, right_size, right_pos)
                self.__createWidget(widget, center_size, center_pos)

    # 创建窗口
    def __createWidget(self,widget:QWidget,size:QSize,point:QPoint):
        widget.setParent(self)
        widget.resize(size)
        widget.raise_()
        widget.move(point)
        widget.show()

    # 动画
    def animation_(self):
        if self.animation_mode == SlideShow.Translation:
            pass
        
        if self.animation_mode == SlideShow.FlatMode:
            pass

    def resizeEvent(self, e:QResizeEvent) -> None:
        self.btnPos()
        self.show_()
        super().resizeEvent(e)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = SlideShow()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor
)

from PyQtGuiLib.core.progressBar import CircularBar


class DurationTimeThread(QThread):
    added = Signal(int)
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


    def run(self) -> None:
        n=1
        while True:
            self.added.emit(n)
            n+=1
            self.msleep(80)
            if n == 100+1:
                break

class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(500,500)

        self.setObjectName("test")
        self.setStyleSheet('''
#test{
background-color: rgb(25, 25, 25);
}
        ''')

        self.th = DurationTimeThread()
        self.th.added.connect(self.test)


        self.cir = CircularBar(self)
        self.cir.resize(150,150)
        self.cir.setVariableLineSegment(CircularBar.Double)
        self.cir.setOuterStyle(CircularBar.CustomDashLine)
        self.cir.setOuterDashPattern([2,3,5,6])
        self.cir.setInnerStyle(CircularBar.DashLine)
        self.cir.setTextSize(15)
        self.cir.move(50,50)
        self.cir.valueChange.connect(lambda v:print("v:",v))

        self.th.start()

    def test(self,n):
        self.cir.setValue(n)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/1/411:16
# @author:LX
# @file:statusBar.py
# @software:PyCharm

from PyQtGuiLib.header import (
    is_win_sys,
    is_mac_sys,
    time,
    QThread,
    Signal,
    QWidget,
    QFrame,
    QHBoxLayout,
    QLabel,
    QPushButton,
    QSpacerItem,
    QSizePolicy,
)
'''
    状态栏
'''


# 倒计时类
class CountDownThread(QThread):
    # 计时完成信号
    timeOuted = Signal()

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.time_ = 0

    # 设置时间
    def setTime(self,time:int):
        self.time_ = time

    def run(self) -> None:
        while self.time_:
            self.sleep(1)
            self.time_-=1
        self.timeOuted.emit()


class StatusBar(QFrame):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.h = 30

        self.__parent = None #  type:QWidget

        if args:
            self.__parent = args[0]

        # 本地时间
        self.format = "%Y-%m-%d %H:%M:%S"
        self.local_time = time.strftime(self.format, time.localtime())
        self.l_time = QLabel(self.local_time)

        # 创建倒计时类
        self.cd = CountDownThread()

        self.__hlay = QHBoxLayout(self)
        self.__hlay.setContentsMargins(6,0,6,0)
        if is_win_sys:
            self.__hlay.setSpacing(6)
        if is_mac_sys:
            self.__hlay.setSpacing(6*3)

        self.hSpacer = QSpacerItem(704, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.updateStatusSize()

        self.startTimer(1000)

    # 设置时间格式
    def setTimeFormat(self,format:str="%Y-%m-%d %H:%M:%S"):
        self.format = format

    def setParent(self, parent:QWidget) -> None:
        self.__parent = parent
        super().setParent(parent)

        if self.__parent is not None:
            self.move(0,self.__parent.height()-self.h)
            self.resize(self.__parent.width(),self.h)

    def __addSpacer(self):
        self.__hlay.addItem(self.hSpacer)
        self.__hlay.removeItem(self.hSpacer)
        self.__hlay.addItem(self.hSpacer)

    # 添加文本
    def addText(self,text:str,style:str=None,duration:int=0):
        '''
            text:文本
            style:样式
            duration:持续时间后消失
        '''
        l = QLabel(text)
        if style:
            l.setStyleSheet(style)

        self.__hlay.addWidget(l)
        self.__addSpacer()

        # 移除布局
        def _del():
            self.__hlay.removeWidget(l)
            l.deleteLater()  # 销毁自己

        if duration > 0:
            self.cd.setTime(duration)
            self.cd.timeOuted.connect(lambda :_del())
            self.cd.start()

    # 添加按钮
    def addButton(self,text:str,style:str=None,callback=None):
        btn = QPushButton(text)
        if style:
            btn.setStyleSheet(style)
        if callback:
            btn.clicked.connect(callback)
        self.__hlay.addWidget(btn)
        self.__addSpacer()

    # 添加widget
    def addWidget(self,widget:QWidget,style:str=None):
        if style:
            widget.setStyleSheet(style)
        self.__hlay.addWidget(widget)
        self.__addSpacer()

    # 添加时间
    def addTime(self,style:str=None):
        if style:
            self.l_time.setStyleSheet(style)
        self.__hlay.addWidget(self.l_time)
        self.__addSpacer()

    def updateStatusSize(self):
        self.move(0, self.__parent.height() - self.h)
        self.resize(self.__parent.width(), self.h)

    def timerEvent(self,e) -> None:
        self.local_time = time.strftime(self.format, time.localtime())
        self.l_time.setText(self.local_time)